## TODO: Check nearest phoneme

currWD <- getwd()
# source("fct.encode.R")

require(dplyr, quietly=TRUE, warn.conflicts=FALSE)
require(tidyr, quietly=TRUE, warn.conflicts=FALSE)
require(stringr, quietly=TRUE, warn.conflicts=FALSE)


compute_errors <- function(acts){
	## output and hidden activations
	epoch <- unique(acts$epoch)
	numUnits <- as.integer(levels(factor(acts$numUnits)))
	numExamples <- max(acts$example)
	output <- filter(acts,group=="output")
	# output$act_bin <- ifelse(output$act<0.5,0,1)
	hidden <- filter(acts,group=="hidden")
	# outputm <- matrix(output$act,nrow=numExamples, byrow=TRUE)
	outputm <- matrix(output$act,nrow=numExamples, byrow=TRUE)
	targetm <- matrix(output$target,nrow=numExamples, byrow=TRUE)

	## Decode phonemes
	# outphon <- decode_activations_nearest(output$act)
	startt <- proc.time()
	if(nrow(output)<100000) outphon <- decode_activations_nearest(output$act) else outphon <- decode_activations_nearest_parallel(output$act)
	proc.time() - startt

	outphonm <- matrix(outphon, nrow=numExamples, byrow=TRUE)
	targetphon <- decode_activations(output$target)
	targetphonm <- matrix(targetphon, nrow=numExamples, byrow=TRUE)

	## Compute errors
	error <- (targetm - outputm)^2
	# Cross-entropy
	t <- targetm
	t[t==0] <- 0.0001
	t[t==1] <- 0.9999
	o <- outputm
	o[o==0] <- 0.0001
	o[o==1] <- 0.9999
	xentropy <- t * log(t/o) + (1-t) * log((1-t)/(1-o))
	
	phonAcc <- outphonm==targetphonm
	
	dat <- tbl_df(data.frame(Example=1:numExamples))
	dat$Epoch <- epoch
	dat$TargetString <- unite(data.frame(targetphonm), Word, 1:ncol(outphonm), sep='')$Word
	dat$OutputString <- unite(data.frame(outphonm), Word, 1:ncol(outphonm), sep='')$Word
	# dat$UnitError <- apply(error,1, function(x) sqrt(mean(x)))
	dat$SSE <- apply(error,1,sum)
	dat$XE <- apply(xentropy, 1, sum)
	dat$Error <- apply(xentropy, 1, sum)
	dat$PhonError <- apply(!phonAcc,1,mean)
	dat$PhonAcc <- dat$PhonError == 0
	# dat$UnitAcc <- dat$UnitError == 0

	dat
}


read_net <- function(itemsFile, resultsFile){
	items <- tbl_df(read.csv(itemsFile, header=TRUE, encoding="UTF-8", stringsAsFactors=FALSE)) 
	items <- items %>% select(ID) %>%
		mutate(Example=1:nrow(items))
	#
	results <- tbl_df(read.table(resultsFile, header=TRUE, stringsAsFactors=FALSE, encoding="UTF-8"))
	results <- left_join(items, results, by="Example")
	results
}

