require(dplyr, quietly=TRUE, warn.conflicts=FALSE)
require(tidyr, quietly=TRUE, warn.conflicts=FALSE)

# for each example:
#  <I total-updates> <I example-number>
#  <I ticks-on-example> <I num-groups>
#  for each tick on the example:
#    <I tick-number> <I event-number>
#    for each WRITE_OUTPUTS group:
#      <I num-units> <B targets?>
#      for each unit:
#        <R output-value> <R target-value>

extract_activations <- function(filename){
	# out <- readLines(paste("test",epoch,".out",sep=""))
	print(filename)
	out <- readLines(filename)
	firstids <- out[1:4]
	numIDs <- 4
	epoch <- as.integer(unlist(strsplit(out[1],' '))[1])
	numGroups <- as.integer(unlist(strsplit(out[2],' '))[2])
	numUnits <- NULL
	numUnits[1] <- as.integer(unlist(strsplit(out[4],' '))[1])
	#' Correct for alternative representation
	if(length(unlist(strsplit(out[1],' '))) == 4){
		numIDs <- 3
		numGroups <- as.integer(unlist(strsplit(out[1],' '))[4])
		numUnits[1] <- as.integer(unlist(strsplit(out[3],' '))[1])
	}

	groupstarts <- NULL
	groupstarts[1] <- numIDs
	for(g in 2:numGroups){
		numUnits[g] <- as.integer(unlist(strsplit(out[numIDs+numUnits[g-1]+1],' '))[1])
		groupstarts[g] <- groupstarts[g-1]+numUnits[g-1]+1
	}

	linesperexample <- sum(numUnits)+(numIDs+1)
	examplestarts <- seq(1,length(out),linesperexample)
	numExamples <- length(examplestarts)
	numUnits <- rep(numUnits,numExamples)
	groupstarts <- sort(as.vector(sapply(groupstarts, function(x) seq(x,length(out), linesperexample))))
	idlines <- sort(c(examplestarts,examplestarts+1,examplestarts+2,groupstarts))

	hasTarget <- as.integer(gsub("([0-9]+) ([0-9]+)","\\2", out[groupstarts]))
	hasTarget <- ifelse(hasTarget==1, T, F)
	groups <- ifelse(hasTarget, "output", "hidden")
	acts <- gsub("([0-9e.-]+) ([01]?)","\\1", out[-idlines])
	actsWithTarget <- grep("[0-9e.-]+ [01]$", out[-idlines])
	targets <- as.integer(gsub("([0-9e.-]+) ([01])$","\\2", out[-idlines][actsWithTarget]))
	# exNum <- as.integer(gsub("([0-9]+) ([0-9]+)","\\2", out[examplestarts]))+1
	exNum <- as.integer(str_match(out[examplestarts], "^[0-9]+ ([0-9]+)")[,2])+1

	summary(factor(targets))
	summary(as.double(acts))
	summary(exNum)
	summary(numGroups)
	summary(factor(numUnits))
	summary(hasTarget)
	summary(factor(groups))

	dat <- tbl_df(data.frame(act=as.double(acts)))
	dat$epoch <- epoch
	dat$example <- rep(rep(exNum,each=2), times = numUnits)
	dat$group <- rep(groups, times = numUnits)
	dat$numUnits <- rep(numUnits, times = numUnits)
	dat$hasTarget <- rep(hasTarget, times = numUnits)
	dat$target <- NA
	dat$target[dat$hasTarget] <- targets

	dat
}



extract_all_epochs <- function(prefix, path="output"){
	files <- list.files(path=path, pattern=paste(prefix,'\\.\\d+\\.out',sep=''))
	dat <- NULL
	for(f in files){
		filename <- paste(path,f,sep='/')
		dat <- bind_rows(dat, extract_activations(filename))
	}
	dat
}


