
# library(multidplyr) # parallel processing
library(parallel, quietly=TRUE, warn.conflicts=FALSE)
library(doParallel, quietly=TRUE, warn.conflicts=FALSE) 
require(tidyr, quietly=TRUE, warn.conflicts=FALSE)
require(dplyr, quietly=TRUE, warn.conflicts=FALSE)
require(readr, quietly=TRUE, warn.conflicts=FALSE)
require(stringr, quietly=TRUE, warn.conflicts=FALSE)
require(ggplot2, quietly=TRUE, warn.conflicts=FALSE)

source("fct.extract_activations.R")
source("fct.compute_errors2.R")
source("fct.encode.R")

args = commandArgs(trailingOnly=TRUE)



process <- function(filename){
	# starttime <- proc.time()
	# print(filename)
	acts <- extract_activations(filename)
	errors <- compute_errors(acts)
	# proc.time() - starttime
	return(errors)

	# ez <- gzfile(resfile, open="ab")
	# if(file.size(resfile)==0) {
	# 	write.table(errors, ez, col.names=TRUE, row.names=FALSE, append=FALSE)
	# } else {
	# 	write.table(errors, ez, col.names=FALSE, row.names=FALSE, append=TRUE)
	# }
	# close(ez)

	# summ <- summarise_each(errors[,-c(1,3:4)], funs(mean))

	# if(!file.exists(summfile)) {
	# 	write.table(summ, summfile, append=FALSE, col.names=TRUE, row.names=FALSE)
	# } else {
	# 	write.table(summ, summfile, append=TRUE, col.names=FALSE, row.names=FALSE)
	# }
}




process_net <- function(path, files, net){
	# starttime <- proc.time()
	# print(net)
	netfiles <- files[str_detect(files, str_c(net, '\\.'))]
	# errors <- NULL
	errors <- foreach(f = netfiles, .combine = rbind) %do% {
		filename <- paste(path,f,sep='/')
		# netname <- str_match(f, "^([^.]+)\\.")[2]
		actsfile <- str_replace(filename, 'out.gz', 'acts.txt.gz')
		# resfile <- str_replace(filename, '\\.\\d+\\.out.gz', '.results.txt.gz')
		# summfile <- str_replace(filename, '\\.\\d+\\.out.gz', '.summary.txt')
		process(filename)
	}

	resfile <- str_c(path,"/",net, ".results.txt.gz")
	ez <- gzfile(resfile, open="ab")
	write.table(errors, ez, col.names=TRUE, row.names=FALSE, append=FALSE)
	close(ez)

	summfile <- str_c(path,"/",net, ".summary.txt")
	summ <- errors[,-c(1,3:4)] %>% group_by(Epoch) %>% summarise_each(funs(mean))
	write.table(summ, summfile, append=FALSE, col.names=TRUE, row.names=FALSE)
	# proc.time() - starttime
}


process_files_parallel <- function(path, cores, maxNetworks, maxEpoch, export=ls()){
	files <- list.files(path=path, pattern=".*\\.\\d+\\.out.gz")
	# Select epochs
	files <- files[as.numeric(str_match(files, '\\.(\\d+)\\.out')[,2])<=maxEpoch]
	files <- files[as.numeric(str_match(files, '^[^.\\d]+(\\d\\d?)\\.')[,2])<=maxNetworks]
	unlink(str_c(path,'/*.results.txt.gz'))
	unlink(str_c(path,'/*.summary.txt'))

	cluster <- makeCluster(cores)
	registerDoParallel(cluster)  
	# getDoParWorkers()

		# foreach(f = files, .export=c('process', 'extract_activations', 'compute_errors', 'decode_activations', 'decode_activations_nearest2', 'phonchunk_activations', 'decode_phoneme', 'get_slotsizes'), .packages=c('tidyverse', 'stringr', 'stringdist')) %dopar% {

	nets <- str_match(files, "^([^.]+\\.[^.]+)\\.")[,2] %>% unique() %>% sample()

	# start <- proc.time()
	# results = foreach(n = nets, .export=c('process_net', 'process', 'extract_activations', 'compute_errors', 'decode_activations', 'decode_activations_nearest2', 'phonchunk_activations', 'decode_phoneme', 'get_slotsizes'), .packages=c('tidyverse', 'stringr', 'stringdist', 'foreach')) %dopar% {
	results = foreach(n = nets, .export=export, .packages=c('tidyr', 'dplyr', 'stringr', 'stringdist', 'foreach')) %dopar% {
		# netfiles <- files[str_detect(files, str_c(n, '\\.'))]
		# foreach(f = netfiles) %do% {
		# 	filename <- paste(path,f,sep='/')
		# 	netname <- str_match(f, "^([^.]+)\\.")[2]
		# 	actsfile <- str_replace(filename, 'out.gz', 'acts.txt.gz')
		# 	resfile <- str_replace(filename, '\\.\\d+\\.out.gz', '.results.txt.gz')
		# 	summfile <- str_replace(filename, '\\.\\d+\\.out.gz', '.summary.txt')
		# 	process(filename, actsfile, resfile, summfile)
		# source("script.summarise.R")
		# }
		process_net(path, files, n)
	}
	# proc.time() - start # End clock
	stopCluster(cluster) # Stop cluster

}



summarise_net <- function(netname, path){
	# train <- read.table(str_c(path,"/",netname,".train.summary.txt"), header=TRUE)
	test <- read.table(str_c(path,"/",netname,".test.summary.txt"), header=TRUE)
	holdout <- read.table(str_c(path,"/",netname,".holdout.summary.txt"), header=TRUE)
	test$Set <- "test"
	# train$Set <- "train"
	holdout$Set <- "holdout"

	# dat <- bind_rows(test,train,holdout)
	dat <- bind_rows(test,holdout)
	dat <- gather(dat, Measure, Value, c(PhonError, SSE, PhonAcc))

	(p <- ggplot(filter(dat, Epoch>1000 & Measure%in%c("PhonAcc", "SSE")), aes(Epoch, Value, color=Set, shape=Set, group=Set)) +
	# geom_line(stat='summary', fun.data="mean_se") +
	# geom_point(stat='summary') +
  # stat_summary(fun.data = "mean_se") +
		geom_line() +
		geom_point() +
		scale_x_sqrt() +
		ylim(0,1) +
		facet_grid(Measure~., scales='free')
		)

	print(str_c(path,"/summary.",netname,".pdf"))
	ggsave(p, file=str_c(path,"/summary.",netname,".pdf"))
}

summarise_results <- function(path){
	files <- list.files(path=path, pattern=".*\\.\\d+\\.out.gz")
	netnames <- unique(str_match(files, "^([^.]+)\\.")[,2])
	for(n in netnames){
		netname <- n
		# path <- prefix
		# try(source("script.summarise.R"))
		try(summarise_net(n, path))
	}
}




if(length(args)>0){
	path <- args[1]
	if(length(args)>1) cores <- args[2] else cores <- detectCores()
	if(length(args)>2) maxNetworks <- as.numeric(args[3]) else maxNetworks <- 100
	if(length(args)>3) maxEpoch <- args[4] else maxEpoch <- 10e6
	start <- proc.time()
	process_files_parallel(path, cores, maxNetworks, maxEpoch, ls())
	print(proc.time() - start) # End clock
	summarise_results(path)
}
