#!/usr/bin/env Rscript

#' Excludes test items
#' (So they have a chance to appear in the probabilistic training set.)


# sink("Rout.txt")

netids = 'test'
corpus <- "fullset.encoded.csv"
args = commandArgs(trailingOnly=TRUE)
if(length(args)>0) {
	corpus <- args[1]
	path <- args[2]
	netids <- args[-(1:2)]
}

source("fct.encode.R")

require(dplyr, quietly=TRUE, warn.conflicts=FALSE)
require(tidyr, quietly=TRUE, warn.conflicts=FALSE)
require(stringr, quietly=TRUE, warn.conflicts=FALSE)
options(dplyr.width = Inf)




for(netid in netids){
	fullset <- tbl_df(read.csv(corpus, header=T, encoding="UTF-8", stringsAsFactors=FALSE))
	
	print(netid)
	unlink(str_c(path,"/",netid,".*"))

	#' Exclude one token per verb of a certain full-paradigm set
	completes <- fullset %>% filter(Testitem) %>% select(Lemma) %>% distinct()
	pns <- fullset %>% filter(Testitem) %>% with(unique(PN))
	nLemmas <- nrow(completes)
	netnum <- str_match(netid, "([0-9]+)$")[1] %>% as.numeric()
	set.seed(8763)
	lemmaOrder <- sample(nLemmas, nLemmas)
	# dropped <- sample(c("sg1","sg2","sg3","pe4","pl2"), nrow(completes), replace=TRUE)
	dropped <- rep(pns, len=nLemmas)
	if(netnum > 1) dropped <- c(dropped[-(1:(netnum-1))], dropped[(1:(netnum-1))])
	novelTokens <- data.frame(completes[lemmaOrder,], dropPN=dropped)
	novelTokens$dropPN <- as.character(novelTokens$dropPN)
	fullset <- left_join(fullset, novelTokens) %>% mutate(dropPN = ifelse(is.na(dropPN), "nodrop", dropPN)) %>% mutate(trained=ifelse(PN==dropPN, FALSE, TRUE))


#' Subsets
	trainset <- filter(fullset, trained)
	testset <- filter(fullset, Testitem)
	novelset <- filter(fullset, !trained)



	#' Add untrained items to testset
	# testset <- filter(fullset, Testitem | !trained)


	#'
	#' Write example files
	#' 
	trainset <- arrange(trainset, Subclass, Lemma, PN)
	testset <- arrange(testset, Subclass, Lemma, PN)
	novelset <- arrange(novelset, Subclass, Lemma, PN)

	write.csv(trainset, file=str_c(path,'/',netid,'.trainset','.csv'), fileEncoding='UTF-8', row.names=FALSE)
	write.csv(testset, file=str_c(path,'/',netid,'.testset','.csv'), fileEncoding='UTF-8', row.names=FALSE)
	write.csv(novelset, file=str_c(path,'/',netid,'.novelset','.csv'), fileEncoding='UTF-8', row.names=FALSE)

	writeExamples(trainset, str_c(path,'/',netid,'.trainset','.ex'))
	writeExamples(testset, str_c(path,'/',netid,'.testset','.ex'))
	writeExamples(novelset, str_c(path,'/',netid,'.novelset','.ex'))

}

# sink()