
set net [getObj name]
set messagefile $net.messages.txt
set procscript script.process.R

# ##
# ## Stem bias
# ##
# for {set i 0} {$i <= 137} {incr i} {
#         connectUnits input:$i output:$i -mean 0.2 -r 0 -t "stembias"
#     }
# freezeWeights -g output -t stembias
# ##
# ##


## load training and testing sets
loadExamples $net.trainset.ex -s train
loadExamples $net.testset.ex -s test
loadExamples $net.novelset.ex -s novel
# loadExamples $net.testset.ex -s test
useTrainingSet train
useTestingSet test
exampleSetMode train PERMUTED
exampleSetMode test ORDERED


changeGroupType "hidden" +WRITE_OUTPUTS 
changeGroupType "hidden" +USE_OUTPUT_HIST
changeGroupType "output" +USE_OUTPUT_HIST
changeGroupType "output" +USE_TARGET_HIST


# int batchSize 	The number of examples over which link error derivatives are accumulated before updating the weights. A value of 0 will make the batch the entire training set.
# setObj batchSize 300

# int numUpdates 	The number of weight updates to be done on the next or current round of training.
setObj numUpdates 1000

setObj reportInterval 10000


###
### TRAINING PARAMETERS
###

## Leaning rate: 0.005 (MSJ), 0.05, 0.1 (PM93)
# Scales the weight deltas (changes):
# setObj learningRate 0.1 

## Momentum: 0.0 (PM93), 0.1, 0.9
# Determines the contribution of the previous weight delta to the new weight delta. It is used in momentum descent and delta-bar-delta.
# setObj momentum 0.9

## Rand range
setObj randRange 0.1

# real gain 	This is the multiplicative inverse of the temperature used in logistic transfer functions. The gain is stored, rather than the temperature, to avoid divisions. The default is 1.0.
# setObj gain 8

# real targetRadius 	The distance that targets will migrate towards outputs, thus reducing the error. If this is non-zero, the target is effectively a range, rather than a point.
# Unlike the previous parameters, this is used during processing of a batch rather than during weight updates following a batch. If an output unit's activation is within this range of the target, there will be no error. This is done by effectively adjusting the target to be the same as the activation. If the activation is farther from the target than this distance, the effective target will be this much closer to the activation.
setObj targetRadius 0

# real zeroErrorRadius 	The distance from targets within which output units will incur no error.
# This is slightly different than targetRadius. If the output unit's activation and target are within this range, there will be no error. But if the activation is farther from the target than this distance, the effective target will not move. Thus, there will be a discontinuity in the error function at this distance from the target. targetRadius and zeroErrorRadius could reasonably be used together if the zeroErrorRadius is larger.
setObj zeroErrorRadius 0

# trainGroupCrit: This is used during training to decide if the groups' groupCriterionReached() functions have achieved criterion.
setObj trainGroupCrit 0

# testGroupCrit: This is used during testing to decide if the groups' groupCriterionReached() functions have achieved criterion.
setObj testGroupCrit 0

# real criterion 	Training will stop if the total error on a batch falls below this value.
setObj criterion 0

# flag groupCritRequired: 	If this is TRUE, an example will stop (no more events will occur) if an event ends and the group criterion was not reached. This is used primarily for continuous networks. The default is FALSE.
setObj groupCritRequired FALSE

# flag autoPlot 	Indicates whether the network is in "autoPlot" mode, which causes the Unit Viewer plot to be re-arranged whenever the network architecture changes.
# autoPlot


viewUnits
# graphObject error
# graphObject group(3).output

resetNet




proc testSave {set} {
	global messagefile
	global procscript
	set net [getObj name]
	set epoch [getObj totalUpdates]
	set outfile $net.$set.$epoch.out.gz
	set actsfile $net.$set.$epoch.acts.txt.gz
	set resfile $net.$set.results.txt.gz
	set summfile $net.$set.summary.txt
	# if {[file isdirectory output] == 0} { file mkdir output}
	set f [open "$messagefile" "a"]
	openNetOutputFile $outfile
	useTestingSet $set
	exampleSetMode $set ORDERED
	puts $f [test -r]
	closeNetOutputFile
	# exec Rscript $procscript $outfile $actsfile $resfile $summfile $net &
	close $f
	exampleSetMode train PROBABILISTIC
}


proc trainTest {{updates 1000} {max 0} {inc 0}} {
	set epoch [getObj totalUpdates]
	if {$max == 0} {set max [expr $epoch + $updates]}
	for {set u $updates} {$epoch < $max} {incr u $inc} {
		useTrainingSet train
		exampleSetMode train PERMUTED
		# setObj batchSize 1
		train $u
		set epoch [getObj totalUpdates]
		testSave train
		testSave test
		testSave novel
	}
}

proc trainTestAll {{updates 1000} {max 0} {inc 0}} {
	set epoch [getObj totalUpdates]
	if {$max == 0} {set max [expr $epoch + $updates]}
	for {set u $updates} {$epoch < $max} {incr u $inc} {
		useTrainingSet train
		exampleSetMode train PROBABILISTIC
		# setObj batchSize 1
		train $u
		set epoch [getObj totalUpdates]
		testSave train
		testSave test
		testSave novel
	}
}
